import { ListPage } from "./list.po";
import { browser, by, element } from "protractor";
import { CalculatorPage } from "./calculator.po";
import { AppPage } from "./app.po";
import { PreferencesPage } from "./preferences.po";
import { Navbar } from "./navbar.po";

/**
 * Check that created/cloned structures have empty fields when
 * "empty fields on calculator creation" is enabled
 */
describe("ngHyd - check that created/cloned structures have empty fields - ", () => {
    let prefPage: PreferencesPage;
    let listPage: ListPage;
    let calcPage: CalculatorPage;
    let navBar: Navbar;

    beforeAll(() => {
        prefPage = new PreferencesPage();
        listPage = new ListPage();
        calcPage = new CalculatorPage();
        navBar = new Navbar();
        browser.manage().window().setPosition(2000, 30);
    });

    beforeEach(async () => {
        // enable evil option "empty fields on module creation"
        await prefPage.navigateTo();
        await browser.sleep(200);
        await prefPage.enableEvilEmptyFields();
        await browser.sleep(200);
    });

    async function setup() {
        // start page
        await navBar.clickNewCalculatorButton();
        await browser.sleep(200);

        // open structures calculator
        await listPage.clickMenuEntryForCalcType(8);
        await browser.sleep(200);
    }

    /**
     * check that a input set is in a given (empty/filled) state
     * @param inputIds id of the inputs to check
     * @param emptys empty/not empty state array
     */
    async function checkFields(inputIds: string[], emptys: boolean[]) {
        let n = 0;
        for (const id of inputIds) {
            const inp = await calcPage.getInputById(id);
            const txt = await inp.getAttribute("value");
            expect(txt === "").toEqual(emptys[n]);
            n++;
        }
    }

    it("when a structure calculator is created", async () => {
        await setup();

        // check 1st structure empty fields
        const inputIds = ["Q", "Z1", "Z2", "0_ZDV", "0_L", "0_W", "0_CdGR"];
        const emptys = [true, true, true, true, true, true, false];
        await checkFields(inputIds, emptys);

        // change 1st structure type to rectangular weir
        const structSelect = calcPage.getSelectById("select_structure");
        await calcPage.changeSelectValue(structSelect, 1);
        await browser.sleep(200);

        // check 1st structure empty fields
        const inputIds2 = ["Q", "Z1", "Z2", "0_ZDV", "0_L", "0_CdWR"];
        const emptys2 = [true, true, true, true, true, false];
        await checkFields(inputIds2, emptys2);
    });

    it("when a structure is added", async () => {
        await setup();

        // add structure
        const addStruct = calcPage.getAddStructureButton();
        await addStruct.click();

        // check 2nd structure empty fields
        const inputIds = ["1_ZDV", "1_L", "1_W", "1_CdGR"];
        const emptys = [true, true, true, false];
        await checkFields(inputIds, emptys);
    });

    it("when a structure is copied", async () => {
        await setup();

        // copy structure
        const addStruct = calcPage.getCopyStructureButton();
        await addStruct.click();

        // check 2nd structure empty fields
        const inputIds = ["1_ZDV", "1_L", "1_W", "1_CdGR"];
        const emptys = [true, true, true, false];
        await checkFields(inputIds, emptys);
    });

    it("when a modified structure is copied (type)", async () => {
        await setup();

        // change 1st structure type to rectangular weir
        const structSelect = calcPage.getSelectById("select_structure");
        await calcPage.changeSelectValue(structSelect, 1);
        await browser.sleep(200);

        // copy structure
        const addStruct = calcPage.getCopyStructureButton();
        await addStruct.click();
        await browser.sleep(200);

        // change 2nd structure type to rectangular gate
        const selects = await element.all(by.css("mat-select#select_structure"));
        const structSelect2 = selects[1];
        await calcPage.changeSelectValue(structSelect2, 4);
        await browser.sleep(200);

        // check empty fields
        const inputIds = ["1_ZDV", "1_L", "1_W", "1_CdGR"];
        const emptys = [true, true, true, false];
        await checkFields(inputIds, emptys);
    });

    it("when a modified structure is copied (input)", async () => {
        await setup();

        // fill 
        const inp = calcPage.getInputById("0_ZDV");
        await inp.clear();
        await inp.sendKeys("1");

        // copy structure
        const addStruct = calcPage.getCopyStructureButton();
        await addStruct.click();
        await browser.sleep(200);

        // check empty fields
        const inputIds = ["1_ZDV", "1_L", "1_W", "1_CdGR"];
        const emptys = [false, true, true, false];
        await checkFields(inputIds, emptys);
    });

    it("except for discharge coefficient when a Larinier weir is used", async () => {
        await setup();

        // change 1st structure type to rectangular weir
        const structSelect = calcPage.getSelectById("select_structure");
        await calcPage.changeSelectValue(structSelect, 1);
        await browser.sleep(200);

        // change discharge law to Larinier
        const dischargeSelect = calcPage.getSelectById("select_loidebit");
        await calcPage.changeSelectValue(dischargeSelect, 3);
        await browser.sleep(200);

        // check empty fields
        const inputIds = ["0_ZDV", "0_L", "0_CdWSL"];
        const emptys = [true, true, true];
        await checkFields(inputIds, emptys);
    });
});
