import { Component, Input } from "@angular/core";

import { Result, cLog, Message, MessageCode, MessageSeverity, MRCInclination } from "jalhyd";

import { fv } from "../../../app/util";

import { CalculatorResults } from "../../results/calculator-results";
import { NgParameter } from "../../formulaire/elements/ngparam";
import { ApplicationSetupService } from "../../services/app-setup.service";
import { I18nService } from "../../services/internationalisation.service";
import { MacrorugoCompoundResults } from "../../results/macrorugo-compound-results";
import { ResultsComponentDirective } from "../fixedvar-results/results.component";

@Component({
    selector: "macrorugo-compound-results",
    templateUrl: "./macrorugo-compound-results.component.html",
    styleUrls: [
        "./macrorugo-compound-results.component.scss"
    ]
})
export class MacrorugoCompoundResultsComponent extends ResultsComponentDirective {

    /** résultats non mis en forme */
    private _mrcResults: MacrorugoCompoundResults;

    constructor(
        private appSetupService: ApplicationSetupService,
        private i18nService: I18nService,
    ) {
        super();
    }

    @Input()
    public set results(rs: CalculatorResults[]) {
        this._mrcResults = undefined;
        if (rs.length > 0 && rs[0] instanceof MacrorugoCompoundResults) {
            this._mrcResults = rs[0] as MacrorugoCompoundResults;
        }
    }

    private mergeGlobalLog(result: Result, log: cLog) {
        if (result) {
            if (result.hasGlobalLog()) {
                log.addLog(result.globalLog);
            }
            // if no parameter is varying, 1st element log is considered "global"
            if (this.mrcResults.variatedParameters.length === 0) {
                if (result.hasResultElements() && result.resultElement.hasLog()) {
                    log.addLog(result.log);
                }
            }
        }
    }

    /**
     * Returns the number of errors, warnings, infos among children logs
     */
    private logStats(): any {
        const ret = {
            info: 0,
            warning: 0,
            error: 0
        };
        if (this._mrcResults.result && this._mrcResults.result.hasLog()) {
            for (const re of this._mrcResults.result.resultElements) {
                if (re.hasLog()) {
                    for (const m of re.log.messages) {
                        const s = m.getSeverity();
                        switch (s) {
                            case MessageSeverity.INFO:
                                ret.info ++;
                                break;
                            case MessageSeverity.WARNING:
                                ret.warning ++;
                                break;
                            case MessageSeverity.ERROR:
                                ret.error ++;
                                break;
                        }
                    }
                }
            }
        }
        for (const cr of this._mrcResults.childrenResults) {
            if (cr && cr.hasLog()) {
                for (const re of cr.resultElements) {
                    if (re.hasLog()) {
                        for (const m of re.log.messages) {
                            const s = m.getSeverity();
                            switch (s) {
                                case MessageSeverity.INFO:
                                    ret.info ++;
                                    break;
                                case MessageSeverity.WARNING:
                                    ret.warning ++;
                                    break;
                                case MessageSeverity.ERROR:
                                    ret.error ++;
                                    break;
                            }
                        }
                    }
                }
            }
        }
        return ret;
    }

    /*
     * Retourne les logs à afficher dans le composant de log global, au dessus
     * du sélecteur d'itération : messages globaux et / ou résumé des messages
     * spécifiques à chaque ResultElement
     */
    public get globalLog(): cLog {
        const l = new cLog();
        if (this._mrcResults && this.mrcResults.variatedParameters.length > 0) {
            this.mergeGlobalLog(this._mrcResults.result, l);
            // un problème avec la MRC en général / les cloisons, à une étape quelconque ?
            if (
                (this.mrcResults.hasLog)
                && l.messages.length === 0 // existing global messages make generic message below useless
            ) {
                const logStats = this.logStats();
                const m = new Message(MessageCode.WARNING_PROBLEMS_ENCOUNTERED);
                m.extraVar.info = "" + logStats.info; // to avoid displaying fixed number of digits
                m.extraVar.warning = "" + logStats.warning;
                m.extraVar.error = "" + logStats.error;
                l.add(m);
                // l.add(new Message(MessageCode.WARNING_PROBLEMS_ENCOUNTERED));
            }
        } // sinon pas de log global (aucun paramètre ne varie)
        return l;
    }

    /**
     * Retourne les logs à afficher dans le composant de log local, en dessous
     * du sélecteur d'itération : messages concernant l'itération (le ResultElement)
     * en cours
     */
    public get iterationLog(): cLog {
        const l = new cLog();
        if (this._mrcResults) {
            if (this.mrcResults.variatedParameters.length > 0) {
                // A. si un paramètre varie
                const vi = this._mrcResults.variableIndex;
                // log de la MRC pour l'itération en cours
                if (
                    this._mrcResults.result
                    && this._mrcResults.result.hasResultElements()
                    && this._mrcResults.result.resultElements[vi]
                    && this._mrcResults.result.resultElements[vi].hasLog()
                ) {
                    l.addLog(this._mrcResults.result.resultElements[vi].log);
                }
                // logs des enfants pour l'itération en cours
                for (const cr of this._mrcResults.childrenResults) {
                    if (cr && cr.hasResultElements() && cr.resultElements[vi].hasLog()) {
                        l.addLog(cr.resultElements[vi].log);
                    }
                }
            } else {
                // B. si aucun paramètre ne varie
                this.mergeGlobalLog(this._mrcResults.result, l); // faut bien mettre le log global quelque part
                // logs des enfants
                for (const cr of this._mrcResults.childrenResults) {
                    if (cr && cr.hasResultElements() && cr.resultElement.hasLog()) {
                        l.addLog(cr.resultElement.log);
                    }
                }
            }
        }
        return l;
    }

    public get mrcResults() {
        return this._mrcResults;
    }

    public formattedValue(p: NgParameter): string {
        const nDigits = this.appSetupService.displayPrecision;
        return p.getValue().toFixed(nDigits);
    }

    public get hasResults(): boolean {
        return this._mrcResults && this._mrcResults.hasResults;
    }

    public get hasDisplayableResults(): boolean {
        let ret = this._mrcResults && this._mrcResults.hasResults;
        if (
            this._mrcResults
            && this._mrcResults.variatedParameters
            && this._mrcResults.variatedParameters.length > 0
        ) {
            ret = ret
            && this._mrcResults.variableIndex !== undefined
            && this._mrcResults.result.resultElements[this._mrcResults.variableIndex] !== undefined
            && this._mrcResults.result.resultElements[this._mrcResults.variableIndex].ok;
        }
        return ret;
    }

    public get uitextGeneralLogTitle(): string {
        return this.i18nService.localizeText("INFO_TITREJOURNAL_GLOBAL");
    }

    public get uitextLateralInclination(): string {
        return this.i18nService.localizeText("INFO_MACRORUGOCOMPOUND_LINCL");
    }

    public get isInclined(): boolean {
        return (
            this.mrcResults
            && this.mrcResults.result
            && this.mrcResults.result.sourceNub
            && this.mrcResults.result.sourceNub.properties.getPropValue("inclinedApron") === MRCInclination.INCLINED
        );
    }

    public get lateralInclination(): string {
        const lincl = this.mrcResults.result.resultElements[this._mrcResults.variableIndex].values.LIncl;
        return fv(lincl);
    }

}
